@@ -14,6 +14,10 @@ class MissingPrimaryKey(PormException):
     ...
 
 
+class MultiplePrimaryKeys(PormException):
+    ...
+
+
 class MissingTablename(PormException):
     ...
 
@@ -40,3 +44,7 @@ class AlreadyConnected(PormException):
 
 class NotConnected(PormException):
     ...
+
+
+class InvalidSQLiteVersion(PormException):
+    ...
