@@ -9,12 +9,13 @@ from pypika import Order, Query
 from pypika.queries import QueryBuilder
 from pypika.terms import Criterion, Parameter
 
-from p3orm.core import Porm
+from p3orm.core import dialect, driver, querybuilder
 from p3orm.exceptions import (
     InvalidRelationship,
     MissingPrimaryKey,
     MissingRelationship,
     MissingTablename,
+    MultiplePrimaryKeys,
     MultipleResultsReturned,
     NoResultsReturned,
 )
@@ -29,6 +30,9 @@ class Table:
 
     __tablename__: str
 
+    class Meta:
+        meta_table: bool = False
+
     #
     # Magic
     #
@@ -65,14 +69,18 @@ class Table:
         return create_model(cls.__name__, __config__=_TableModelConfig, **factory_model_kwargs)
 
     def __init_subclass__(cls) -> None:
-        if not hasattr(cls, "__tablename__") or cls.__tablename__ is None:
+        if (not hasattr(cls, "__tablename__") or cls.__tablename__ is None) and cls.Meta.meta_table == False:
             raise MissingTablename(f"{cls.__name__} must define a __tablename__ property")
 
         fields = cls._fields()
 
-        if len([f for f in fields if f.pk]) != 1:
+        num_pkeys = len([f for f in fields if f.pk])
+        if num_pkeys == 0:
             raise MissingPrimaryKey(f"{cls.__name__} must have 1 primary key field")
 
+        elif num_pkeys > 1:
+            raise MultiplePrimaryKeys(f"{cls.__name__} has more than 1 primary key field (check parent classes)")
+
     #
     # Introspective methods
     #
@@ -93,13 +101,14 @@ class Table:
     @classmethod
     def _fields(cls, exclude_autogen: Optional[bool] = False) -> List[_PormField]:
         fields: List[_PormField] = []
-        for field_name in cls.__dict__:
-            if isinstance(field := getattr(cls, field_name), _PormField):
-                if field.column_name is None:
-                    field.column_name = field_name
-                    field.name = field_name
-                field.field_name = field_name
-                fields.append(field)
+        for table in cls.__mro__:
+            for field_name in table.__dict__:
+                if isinstance(field := getattr(table, field_name), _PormField):
+                    if field.column_name is None:
+                        field.column_name = field_name
+                        field.name = field_name
+                    field.field_name = field_name
+                    fields.append(field)
 
         if exclude_autogen:
             fields = [f for f in fields if not f.autogen]
@@ -131,7 +140,7 @@ class Table:
     #
     @classmethod
     def from_(cls) -> QueryBuilder:
-        return QueryBuilder().from_(cls.__tablename__)
+        return querybuilder().from_(cls.__tablename__)
 
     @classmethod
     def select(cls, to_select: str = "*") -> QueryBuilder:
@@ -139,11 +148,11 @@ class Table:
 
     @classmethod
     def delete(cls) -> QueryBuilder:
-        return QueryBuilder().delete().from_(cls.__tablename__)
+        return querybuilder().delete().from_(cls.__tablename__)
 
     @classmethod
     def update(cls) -> QueryBuilder:
-        return QueryBuilder().update(cls.__tablename__)
+        return querybuilder().update(cls.__tablename__)
 
     #
     # Queries
@@ -164,7 +173,7 @@ class Table:
 
         query = query.insert(query_params)
 
-        inserted: Optional[Model] = await Porm.fetch_one(cls, with_returning(query), query_args)
+        inserted: Optional[Model] = await driver().fetch_one(cls, with_returning(query), query_args)
 
         if prefetch and inserted:
             [inserted] = await cls.fetch_related([inserted], prefetch)
@@ -180,6 +189,9 @@ class Table:
         prefetch: FetchType = None,
     ) -> List[Model]:
 
+        if not items:
+            return []
+
         columns = cls._fields(exclude_autogen=True)
         columns_count = len(columns)
         query: QueryBuilder = Query.into(cls.__tablename__).columns(*columns)
@@ -190,7 +202,7 @@ class Table:
             query_params = [Parameter(f"${page * columns_count + offset + 1}") for offset in range(columns_count)]
             query = query.insert(query_params)
 
-        inserted = await Porm.fetch_many(cls, with_returning(query), query_args)
+        inserted = await driver().fetch_many(cls, with_returning(query), query_args)
 
         if prefetch and inserted:
             inserted = await cls.fetch_related(inserted, prefetch)
@@ -206,11 +218,11 @@ class Table:
         prefetch: FetchType = None,
     ) -> Optional[Model]:
 
-        paramaterized_criterion, query_args = paramaterize(criterion)
+        paramaterized_criterion, query_args = paramaterize(criterion, dialect=dialect())
         query: QueryBuilder = cls.select().where(paramaterized_criterion)
         query = query.limit(1)
 
-        result = await Porm.fetch_one(cls, query.get_sql(), query_args=query_args)
+        result = await driver().fetch_one(cls, query.get_sql(), query_args=query_args)
 
         if result and prefetch:
             [result] = await cls.fetch_related([result], prefetch)
@@ -226,12 +238,12 @@ class Table:
         prefetch: FetchType = None,
     ) -> Model:
 
-        paramaterized_criterion, query_args = paramaterize(criterion)
+        paramaterized_criterion, query_args = paramaterize(criterion, dialect=dialect())
 
         query: QueryBuilder = cls.select().where(paramaterized_criterion)
         query = query.limit(2)
 
-        results = await Porm.fetch_many(cls, query.get_sql(), query_args)
+        results = await driver().fetch_many(cls, query.get_sql(), query_args)
 
         if len(results) > 1:
             raise MultipleResultsReturned(f"Multiple {cls.__name__} were returned when only one was expected")
@@ -261,7 +273,7 @@ class Table:
 
         query_args = None
         if criterion:
-            parameterized_criterion, query_args = paramaterize(criterion)
+            parameterized_criterion, query_args = paramaterize(criterion, dialect=dialect())
             query = query.where(parameterized_criterion)
 
         if order:
@@ -271,7 +283,7 @@ class Table:
         if limit:
             query = query.limit(limit)
 
-        results = await Porm.fetch_many(cls, query.get_sql(), query_args)
+        results = await driver().fetch_many(cls, query.get_sql(), query_args)
 
         if prefetch:
             results = await cls.fetch_related(results, prefetch)
@@ -287,10 +299,10 @@ class Table:
         prefetch: FetchType = None,
     ) -> Model:
 
-        query: QueryBuilder = QueryBuilder().update(cls.__tablename__)
+        query: QueryBuilder = querybuilder().update(cls.__tablename__)
 
         pk = cls._primary_key()
-        parameterized_criterion, query_args = paramaterize(pk == getattr(item, pk.field_name))
+        parameterized_criterion, query_args = paramaterize(pk == getattr(item, pk.field_name), dialect=dialect())
 
         for i, field in enumerate(cls._fields()):
             field: _PormField
@@ -299,7 +311,7 @@ class Table:
 
         query = query.where(parameterized_criterion)
 
-        updated = await Porm.fetch_one(cls, with_returning(query), query_args)
+        updated = await driver().fetch_one(cls, with_returning(query), query_args)
 
         if prefetch and updated:
             [updated] = await cls.fetch_related([updated], prefetch)
@@ -309,13 +321,13 @@ class Table:
     @classmethod
     async def delete_where(cls: Union[Type[Model], Table], /, criterion: Criterion) -> List[Model]:
 
-        query: QueryBuilder = QueryBuilder().delete()
+        query: QueryBuilder = querybuilder().delete()
         query = query.from_(cls.__tablename__)
 
-        parameterized_criterion, query_args = paramaterize(criterion)
+        parameterized_criterion, query_args = paramaterize(criterion, dialect=dialect())
 
         query = query.where(parameterized_criterion)
-        return await Porm.fetch_many(cls, with_returning(query), query_args)
+        return await driver().fetch_many(cls, with_returning(query), query_args)
 
     @classmethod
     async def _load_relationships_for_items(
@@ -356,11 +368,12 @@ class Table:
             )
 
             paramaterized_criterion, query_args = paramaterize(
-                relationship_table._field(relationship.foreign_column).isin(keys)
+                relationship_table._field(relationship.foreign_column).isin(keys),
+                dialect=dialect(),
             )
             sub_items_query: QueryBuilder = relationship_table.select().where(paramaterized_criterion)
 
-            sub_items = await Porm.fetch_many(relationship_table, sub_items_query.get_sql(), query_args)
+            sub_items = await driver().fetch_many(relationship_table, sub_items_query.get_sql(), query_args)
 
             if relationship.relationship_type == RelationshipType.foreign_key:
                 sub_items_map = {getattr(i, relationship.foreign_column): i for i in sub_items}
